@@ -59,7 +59,7 @@ class _TreeVisitor(TagTemplateParserVisitor):
     def visitTerminal(self, node):
         if node.getSymbol().type not in ARGUMENT_VALUE_TYPES:
             return IGNORED_TERMINAL
-        return self.defaultResult()
+        raise NotImplementedError()
 
     def aggregateResult(
         self, pattern: List[PatternElement], element: PatternElement
@@ -76,7 +76,7 @@ class _TreeVisitor(TagTemplateParserVisitor):
     ) -> List[TagPlaceholder]:
         if ctx.errorNonTagInPipeList:
             non_tag_symbol = ctx.errorNonTagInPipeList.children[0].symbol
-            raise TagTemplateSyntaxError(
+            raise TemplateSyntaxError(
                 location_from_symbol(non_tag_symbol),
                 message=f"non-tag in the pipe list",
             )
@@ -96,19 +96,27 @@ class _TreeVisitor(TagTemplateParserVisitor):
         return Pattern(pattern_elements)
 
     def visitTag(self, ctx: TagTemplateParser.TagContext) -> TagPlaceholder:
+        category_name = None
+        if ctx.categoryId:
+            category_name = ctx.categoryId.text
         if ctx.errorMissingTagId:
-            raise TagTemplateSyntaxError(
+            raise TemplateSyntaxError(
                 location_from_symbol(ctx.errorMissingTagId),
                 message=f"missing tag name",
             )
-        tag_name = ctx.TAG_ID().getText()
-        if ctx.errorNoArgumentList:
-            raise TagTemplateSyntaxError(
-                location_from_symbol(ctx.errorNoArgumentList),
-                message=f"missing argument list for tag '{tag_name}'",
+        if ctx.errorMissingCategoryId:
+            raise TemplateSyntaxError(
+                location_from_symbol(ctx.errorMissingCategoryId),
+                message=f"missing category name",
             )
+        if ctx.errorNoArgumentList:
+            raise TemplateSyntaxError(
+                location_from_symbol(ctx.errorNoArgumentList),
+                message=f"missing argument list for tag '{ctx.errorNoArgumentList.text}'",
+            )
+        tag_name: str = ctx.tagId.text  # type: ignore
         if ctx.errorUnclosedContext:
-            raise TagTemplateSyntaxError(
+            raise TemplateSyntaxError(
                 location_from_symbol(ctx.errorUnclosedContext),
                 message=f"missing closing context bracket for tag '{tag_name}'",
             )
@@ -120,19 +128,19 @@ class _TreeVisitor(TagTemplateParserVisitor):
 
         tag = TagPlaceholder(
             tag_name=tag_name,
+            category_name=category_name,
             args=args,
             kwargs=kwargs,
             context=context,
         )
-        symbol = ctx.TAG_ID().getSymbol()
-        tag.location = location_from_symbol(symbol)
+        tag.location = location_from_symbol(ctx.tagId)
         return tag
 
     def visitArgumentList(
         self, ctx: TagTemplateParser.ArgumentListContext
     ) -> Tuple[List[ArgValue], Mapping[str, ArgValue]]:
         if ctx.errorUnclosedArgumentList:
-            raise TagTemplateSyntaxError(
+            raise TemplateSyntaxError(
                 location_from_symbol(ctx.errorUnclosedArgumentList),
                 message="missing closing argument list bracket",
             )
@@ -201,40 +209,13 @@ class TagTreeBuilder:
 
 class TagTemplateErrorListener(ErrorListener):
     def syntaxError(self, recognizer, offendingSymbol, line, column, msg, e):
-        from pprint import pprint
-
-        # pprint([">>>>>///", recognizer, offendingSymbol, line, column, msg, e])
         error_location = Location(line, column, len(offendingSymbol.text))
         if "extraneous input" in msg or "mismatched input" in msg:
-            raise TagTemplateSyntaxError(
+            raise TemplateSyntaxError(
                 error_location,
                 f"unexpected symbol '{offendingSymbol.text}'",
             )
-        raise TagTemplateSyntaxError(error_location, msg)
-
-    def reportAmbiguity(
-        self, recognizer, dfa, startIndex, stopIndex, exact, ambigAlts, configs
-    ):
-        # print("reportAmbiguity")
-        super().reportAmbiguity(
-            recognizer, dfa, startIndex, stopIndex, exact, ambigAlts, configs
-        )
-
-    def reportAttemptingFullContext(
-        self, recognizer, dfa, startIndex, stopIndex, conflictingAlts, configs
-    ):
-        # print("reportAttemptingFullContext")
-        super().reportAttemptingFullContext(
-            recognizer, dfa, startIndex, stopIndex, conflictingAlts, configs
-        )
-
-    def reportContextSensitivity(
-        self, recognizer, dfa, startIndex, stopIndex, prediction, configs
-    ):
-        # print("reportContextSensitivity")
-        super().reportContextSensitivity(
-            recognizer, dfa, startIndex, stopIndex, prediction, configs
-        )
+        raise TemplateSyntaxError(error_location, msg)
 
 
 class TemplateError(Exception):
@@ -250,15 +231,15 @@ class TemplateError(Exception):
         self.message = message
 
 
-class TagTemplateSyntaxError(TemplateError):
+class TemplateSyntaxError(TemplateError):
     pass
 
 
-class TagTemplateSemanticError(TemplateError):
+class TemplateSemanticError(TemplateError):
     pass
 
 
-class TagError(TagTemplateSemanticError):
+class TagError(TemplateSemanticError):
     tag_name: str
 
     def __init__(self, location: Location, tag_name: str, message: str):
@@ -272,6 +253,30 @@ class UnknownTagError(TagError):
         super().__init__(location, tag_name, "Tag not recognized")
 
 
+class AmbiguousTagNameError(Exception):
+    tag_name: str
+    category_names: List[str]
+
+    def __init__(self, tag_name: str, category_names: List[str]):
+        super().__init__(
+            f"Tag {tag_name!r} is present in multiple categories: {','.join(category_names)}"
+        )
+        self.tag_name = tag_name
+        self.category_names = category_names
+
+
+class AmbiguousTagError(TagError):
+    category_names: List[str]
+
+    def __init__(self, location: Location, tag_name: str, category_names: List[str]):
+        super().__init__(
+            location,
+            tag_name,
+            f"This tag name is present in multiple categories: {','.join(category_names)}",
+        )
+        self.category_names = category_names
+
+
 class ContextMissingError(TagError):
     def __init__(self, location: Location, tag_name: str):
         super().__init__(location, tag_name, f"Context is required for this tag")
@@ -338,12 +343,27 @@ class TagRegistry:
     def find_category(self, category_name: str) -> Optional[TagCategory]:
         return self.category_map.get(category_name, None)
 
-    def find_tag_factory(self, tag_name: str) -> Optional[TagFactory]:
-        for category in self.category_map.values():
-            tag_factory = category.find_tag_factory(tag_name)
-            if tag_factory:
-                return tag_factory
-        return None
+    def find_tag_factory(
+        self, tag_name: str, category_name: Optional[str] = None
+    ) -> Optional[TagFactory]:
+        if category_name is None:
+            found_tag_factories: Dict[str, TagFactory] = {}
+            for category in self.category_map.values():
+                tag_factory = category.find_tag_factory(tag_name)
+                if tag_factory:
+                    found_tag_factories[category.name] = tag_factory
+
+            if not found_tag_factories:
+                return None
+            elif len(found_tag_factories) == 1:
+                return next(iter(found_tag_factories.values()))
+            else:
+                category_names = sorted(list(found_tag_factories.keys()))
+                raise AmbiguousTagNameError(tag_name, category_names)
+        else:
+            if category_name not in self.category_map:
+                return None
+            return self.category_map[category_name].find_tag_factory(tag_name)
 
     def bind(self, pattern: Pattern) -> Pattern:
         return self._rewrite_pattern(pattern)
@@ -360,11 +380,18 @@ class TagRegistry:
         return bound_pattern
 
     def _rewrite_tag_placeholder(self, tag_placeholder: TagPlaceholder) -> TagInstance:
-        tag_factory: Optional[TagFactory] = self.find_tag_factory(
-            tag_placeholder.tag_name
-        )
-        if not tag_factory:
-            raise UnknownTagError(tag_placeholder.location, tag_placeholder.tag_name)
+        try:
+            tag_factory: Optional[TagFactory] = self.find_tag_factory(
+                tag_placeholder.tag_name, tag_placeholder.category_name
+            )
+            if not tag_factory:
+                raise UnknownTagError(
+                    tag_placeholder.location, tag_placeholder.tag_name
+                )
+        except AmbiguousTagNameError as exc:
+            raise AmbiguousTagError(
+                tag_placeholder.location, exc.tag_name, exc.category_names
+            ) from exc
 
         try:
             self.log.debug(
@@ -398,6 +425,7 @@ class TagRegistry:
     def register_category(
         self, category_name: str, description: Optional[str] = None
     ) -> TagCategory:
+        # TODO: This method should receive already build (non-empty) TagCategory
         if self.find_category(category_name) is not None:
             raise ValueError(f"Category '{category_name}' already registered")
         new_category = TagCategory(category_name)
@@ -413,7 +441,12 @@ class TagRegistry:
             category_name = module.__name__
 
         def is_tag_class(klass: type):
-            if not inspect.isclass(klass) or not issubclass(klass, Tag) or klass == Tag:
+            if (
+                not inspect.isclass(klass)
+                or not issubclass(klass, Tag)
+                or inspect.isabstract(klass)
+                or klass == Tag
+            ):
                 return False
             return klass.__name__.endswith(self._tag_class_suffix)
 
@@ -430,5 +463,11 @@ class TagRegistry:
         ):
             if is_pkg:
                 continue
-            module = importlib.import_module(name)
-            self.register_tags_in_module(module)
+            try:
+                self.log.debug(f"Trying to load {name} module")
+                module = importlib.import_module(name)
+                self.register_tags_in_module(module)
+            except NotImplementedError as exc:
+                self.log.warning(f"Module {name} is currently unsupported: {exc}")
+            except Exception as exc:
+                self.log.error(exc, f"Could not load module {name}")
