@@ -1,7 +1,6 @@
 import textwrap
 from abc import ABC, abstractmethod
 from dataclasses import dataclass, field
-from pathlib import Path
 from typing import Any, List, Mapping, Optional, Type
 
 from docstring_parser import parse as parse_docstring
@@ -87,6 +86,7 @@ class TagPlaceholder(PatternElement):
 
     location: Location = field(init=False, compare=False)
     tag_name: str
+    category_name: Optional[str] = None
     context: Optional[Pattern] = None
     args: List[Any] = field(default_factory=list)
     kwargs: Mapping[str, Any] = field(default_factory=dict)
@@ -94,6 +94,8 @@ class TagPlaceholder(PatternElement):
     def __post_init__(self):
         if len(self.tag_name) < 1:
             raise ValueError(f"Invalid tag name: ${repr(self.tag_name)}")
+        if self.category_name is not None and len(self.tag_name) < 1:
+            raise ValueError(f"Invalid category name: ${repr(self.category_name)}")
 
     def process(self, file: File) -> str:
         raise NotImplementedError(
@@ -101,7 +103,7 @@ class TagPlaceholder(PatternElement):
         )
 
 
-class FileNotSupportedError(Exception):  # FIXME: use
+class FileNotSupportedError(Exception):
     """Tag cannot extract value due to invalid file type"""
 
     pass
@@ -186,7 +188,7 @@ class TagFactoryFromClass(TagFactory):
         if self._tag_class.require_context is None:
             context_metavar = "[{...}]"
         elif self._tag_class.require_context is True:
-            context_metavar = "{...}"
+            context_metavar = "{...}"  # TODO: test?
         else:
             context_metavar = ""
         single_line_signature = f"%{self._tag_name}{signature}{context_metavar}"
@@ -244,4 +246,7 @@ class TagInstance(PatternElement):
 
     def process(self, file: File) -> Any:
         context_str = self.context.process(file) if self.context else None
-        return self.tag.process(file, context_str)
+        try:
+            return self.tag.process(file, context_str)
+        except MissingMetadataError:
+            return ""
